@@ -1,11 +1,12 @@
 safesolve.qr.matrix <-
-function(qr.a,B,silent=T,stop.on.error=T) { ## solve.qr with fall-back; qr.a should be a qr object, B a matrix
+function(qr.a,B,silent=TRUE,stop.on.error=TRUE) { ## solve.qr with fall-back; qr.a should be a qr object, B a matrix
   ## there was a 'Matrix' subcode prior to 10/03/2013
     res <- try(solve.qr(qr.a,B),silent=silent)
-    if (class(res)=="try-error") { ##FR->FR omprendre pourquoi c'est utile (systematique pour blackcap... Re: phi -> 0 ? )
-      pivI <- sort.list(qr.a$pivot)
-      solveA <- try(solve(qr.R(qr.a)[,pivI])%*%t(qr.Q(qr.a)),silent=silent)  
+    if (class(res)=="try-error") { ##FR->FR sb systematique qd phi -> 0; slow step.
+      # pivI <- sort.list(qr.a$pivot)  ## inverse perm such as pivI[$pivot]=$pivot[pivI]= identity
+      #solveA <- try(solve(qr.R(qr.a)[,pivI])%*%t(qr.Q(qr.a)),silent=silent)  
       ## solveA is solve(<original 'a' matrix>) using the QR decomp... but this may work when solve.qr fails !
+      solveA <- try(backsolve(qr.R(qr.a),t(qr.Q(qr.a))[qr.a$pivot,]))
       if (class(solveA)=="try-error") {
         if (stop.on.error) {
           mess <- pastefrom("class(solveA)='try-error'.",prefix="(!) From ")
