@@ -1,5 +1,5 @@
 safesolve.qr.vector <-
-function(qr.a,b,silent=T,stop.on.error=T) { ## solve.qr with fall-back; qr.a should be a qr object, b must be a vector
+function(qr.a,b,silent=TRUE,stop.on.error=TRUE) { ## solve.qr with fall-back; qr.a should be a qr object, b must be a vector
   if (class(qr.a)=="sparseQR") { ## pas de 'essai' en var locale !
     ## there was a 'Matrix' subcode prior to 10/03/2013; another try on 11/2013
     res <- qr.coef(qr.a,b)
@@ -10,11 +10,12 @@ function(qr.a,b,silent=T,stop.on.error=T) { ## solve.qr with fall-back; qr.a sho
       ## The following is equivalent to solveA <- try(solve(qr.R(qr.a)[,pivI])%*%t(qr.Q(qr.a)),silent=silent) then return solveA %*% b 
       ## but uses only backward mutliplication of vectors and transpose of vectors  
       res <- t(t(b) %*% (qr.Q(qr.a))) ## not yet res
-      pivI <- sort.list(qr.a$pivot)
-      res <- try(solve(qr.R(qr.a)[, pivI]) %*% res, silent=silent)
+      #pivI <- sort.list(qr.a$pivot) ## inverse perm such as pivI[$pivot]=$pivot[pivI]= identity
+      #res <- try(solve(qr.R(qr.a)[, pivI]) %*% res, silent=silent)
+      res <- try(backsolve(qr.R(qr.a),res[qr.a$pivot]))
       if (class(res)=="try-error") {
         if (stop.on.error) {
-          mess <- pastefrom("class(solveA)='try-error'.",prefix="(!) From ")
+          mess <- pastefrom("class(res)='try-error'.",prefix="(!) From ")
           message(mess)
           stop("More code is needed to handle this... I exit.") ## perhaps recover A by qr.X and solve(A) ?
         } else return(res) ## passes control to calling function
